//
//  GetCommentView.swift
//  SampleApp
//
//  Created by Zay Yar Htun on 8/8/23.
//  Copyright © 2023 David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

struct GetCommentView: View {
    @State private var commentId: String = ""
    @StateObject var viewModel = GetCommentViewModel()
    
    var body: some View {
        Form {
            TextField("Enter Comment Id", text: $commentId)
            
            Section {
                Button {
                    guard !commentId.isEmpty else {
                        viewModel.result = ""
                        return
                    }
                    
                    viewModel.getComment(id: commentId)
                    
                } label: {
                    HStack {
                        Spacer()
                        Text("Get Comment")
                        Spacer()
                    }
                }
            }
            
            Section(header: Text("Here you will see result received from sdk")) {
                VStack(alignment: .leading) {
                    Text("Result")
                        .font(.system(size: 18))
                        .fontWeight(.bold)
                        .padding(.bottom, 12)
                    Text(viewModel.result)
                }
            }
        }
        .navigationTitle("Get Comment")
    }
}

class GetCommentViewModel: ObservableObject {
    @Published var result: String = ""
    
    var token: AmityNotificationToken?
    var commentRepo = AmityCommentRepository(client: AmityManager.shared.client!)
    
    func getComment(id: String) {
        token = commentRepo.getComment(withId: id).observe { [weak self] liveObject, error in
            guard let weakSelf = self else {
                return
            }
            
            if error != nil {
                weakSelf.result = ""
                weakSelf.token?.invalidate()
                return
            }
            
            if let object = liveObject.object {
                var textData: String = ""
                
                if let text = object.data?["text"] as? String, text != "" {
                    textData += text + "\n\n"
                }
                
                let sortedDataTypes = object.dataTypes.map { $0.rawValue }.sorted(by: { $0 < $1 })

                textData += "Comment Id: \(object.commentId)\n\n"
                textData += "DataTypes: \(sortedDataTypes)\n\n"
                textData += "Reference Type: \(object.referenceType.description)\n\n"
                textData += "Reference Id: \(object.referenceId)\n\n"
                textData += "My Reactions: \(object.myReactions.joined(separator: ","))\n\n"
                textData += "Metadata: \(String(describing: object.metadata))\n\n"
                textData += "Children Number: \(object.childrenNumber)\n\n"
                textData += "Is Deleted: \(object.isDeleted)\n\n"
                switch object.target {
                case .community(_, let creatorMember):
                    textData += "TargetType: \(object.target.getStringValue())\n"
                    if let creatorMember {
                        textData += "CommunityUserRole: \(creatorMember.roles)\n"
                    } else {
                        textData += "CommunityUserRole: []\n"
                    }
                case .content, .user:
                    textData += "TargetType: \(object.target.getStringValue())\n"
                default:
                    break
                }
                weakSelf.result = textData
            }
        }
    }
}
