//
//  TestController.swift
//  SampleApp
//
//  Created by Nishan Niraula on 10/15/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import Foundation
import UIKit
import AmitySDK
import Combine
import SwiftUI

// NOTE:
// Ignore stuffs in this class. This class is just for quick testing of sdk stuffs inside sample app.
class TestController: UIViewController {
    let buttonStack = UIStackView(frame: .zero)
    
    let client = AmityManager.shared.client!
    
    var token: AmityNotificationToken?
    
    let userRepo = AmityUserRepository(client: AmityManager.shared.client!)
    let commRepo = AmityCommunityRepository(client: AmityManager.shared.client!)
    let commentRepo = AmityCommentRepository(client: AmityManager.shared.client!)
    let postRepo = AmityPostRepository(client: AmityManager.shared.client!)
    let channelRepo = AmityChannelRepository(client: AmityManager.shared.client!)
    let messageRepo = AmityMessageRepository(client: AmityManager.shared.client!)
    let fileRepo = AmityFileRepository(client: AmityManager.shared.client!)
    let subChannelRepo = AmitySubChannelRepository(client: AmityManager.shared.client!)
    let reactionRepo = AmityReactionRepository(client: AmityManager.shared.client!)
    let userPresenceRepo = AmityUserPresenceRepository(client: AmityManager.shared.client!)
    let channelPresenceRepo = AmityChannelPresenceRepository(client: AmityManager.shared.client!)
    
    var disposeBag: Set<AnyCancellable> = []
    
    var label = UILabel()
    
    override func loadView() {
        view = UIView()
        view.backgroundColor = .white
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        AmityLog.logLevel = .all
        
        
        setupViews()
    }
    
    let diffQueue = DispatchQueue(label: "my.good.queue")
    
    var channels = [AmityChannel]()
    
    var channelCollection: AmityCollection<AmityChannel>?
    
    @objc func onButton1Tap() {
        
        let count = channelRepo.totalDefaultSubChannelUnreadCount
        
        let channelQuery = AmityChannelQuery()
        channelQuery.types = [AmityChannelQueryType.community]
        channelQuery.filter = .userIsMember
        
        // Observe snapshot
        channelCollection = channelRepo.getChannels(with: channelQuery)
        channelCollection?.$snapshots.sink { completion in
            print(">>>> Snapshot Completion Triggered >>>>>")
        } receiveValue: { channels in
            print(">>>> Sink Triggered >>>>>>")
//            channels.forEach { item in
//                print(item.channelId)
//            }
            self.accessInBackgroundThread(channels: channels)
            
        }.store(in: &disposeBag)
        
        let status = channelCollection!.dataStatus
        print("Data Status Direct: \(status)")
        
        // Observe
        channelCollection?.$dataStatus.sink(receiveCompletion: { completion in
            print(">>>> Data Status Completion")
        }, receiveValue: { status in
            print(">>>> Data Status: \(status.description)")
        }).store(in: &disposeBag)
    }
    
    func accessInBackgroundThread(channels: [AmityChannel]) {
        print(">>>> Accessing in background thread")
        diffQueue.async {
            channels.forEach { item in
                print(item.channelId)
            }
        }
    }
    
    @objc func onButton2Tap() {
        channelCollection?.nextPage()
    }
    
    var createdChannel: AmityChannel?
    
    @MainActor
    @objc func onButton3Tap() {
        // Create Channel
        let commChannelBuilder = AmityCommunityChannelBuilder()
        commChannelBuilder.channelType = .community
        commChannelBuilder.setDisplayName("Nishan-Swiftui-test")
        commChannelBuilder.setMetadata(["meta": "data"])
        
        Task {
            let channel = try? await channelRepo.createChannel(with: commChannelBuilder)
            self.createdChannel = channel
            print("Channel Created!")
        }
    }
    
    @MainActor
    @objc func onButton4Tap() {
        guard let channelId = createdChannel?.channelId else {
            print("No channel id found")
            return
        }
        
        let updateBuilder = AmityChannelUpdateBuilder(channelId: channelId)
        updateBuilder.setDisplayName("Update-Nishan-SwiftUI-test")
        
        Task {
            let channel = try? await channelRepo.editChannel(with: updateBuilder)
            print("Channel Updated!")
        }
    }
}

struct TestView: View {
    
    @StateObject var viewModel = TestViewModel()
    
    var body: some View {
        NavigationView {
            VStack {
                Text("Hey! LiveCollection works with SwiftUI natively now")
                
                NavigationLink {
                    ChannelListView(collection: viewModel.getNewCollection())
                } label: {
                    Text("Navigate")
                }
                
                ChannelListView(collection: viewModel.collection)
            }
        }
    }
}

struct ChannelListView: View {
    
    @ObservedObject var collection: AmityCollection<AmityChannel>
    
    var viewModel = TestViewModel()
    
    var body: some View {
        VStack {
            VStack(alignment: .leading) {
                Text("Data Status: \(collection.dataStatus.description)")
            
                HStack {
                    Text("Loading Status: \(collection.loadingStatus.description)")
                    
                    Spacer()
                    
                    ProgressView()
                        .progressViewStyle(.circular)
                        .opacity(collection.loadingStatus == .loading ? 1 : 0)
                }
                
                HStack {
                    Button {
                        viewModel.createChannel()
                    } label: {
                        Text("Create")
                    }

                    Button {
                        viewModel.updateChannel()
                    } label: {
                        Text("Update")
                    }

                }
            }
            .padding(.horizontal)
            
            if let error = collection.error {
                Text("Unwrapped Error: \(error.localizedDescription)")
            }
            
            Text("Not Unrapped Error: \(collection.error?.localizedDescription ?? "-")")
                        
            List {
                ForEach(collection.snapshots, id: \.channelId) { channel in
                    Text(getChannelDisplayName(channel: channel))
                        .onAppear {
                            if let lastChannel = collection.snapshots.last, lastChannel.channelId == channel.channelId {
                                loadMoreChannels()
                            }
                        }
                }
            }
        }
    }
    
    func loadMoreChannels() {
        if collection.hasNext {
            print("Loading more channels: \(collection.loadingStatus.description)")
            collection.nextPage()
            print("Loading more channels: \(collection.loadingStatus.description)")
        }
    }
    
    func getChannelDisplayName(channel: AmityChannel) -> String {
        var displayName = channel.displayName?.trimmingCharacters(in: .whitespacesAndNewlines) ?? ""
        displayName = displayName.isEmpty ? channel.channelId : displayName
        return displayName
    }
}

class TestViewModel: ObservableObject {
    
    let channelRepo = AmityChannelRepository(client: AmityManager.shared.client!)
    @Published var collection: AmityCollection<AmityChannel>
    
    var createdChannel: AmityChannel?
    
    init() {
        let channelQuery = AmityChannelQuery()
        channelQuery.types = [AmityChannelQueryType.community]
        channelQuery.filter = .userIsMember
        
        collection = channelRepo.getChannels(with: channelQuery)
    }
    
    @MainActor
    @objc func createChannel() {
        // Create Channel
        let commChannelBuilder = AmityCommunityChannelBuilder()
        commChannelBuilder.channelType = .community
        commChannelBuilder.setDisplayName("\(Int.random(in: 0..<100))-Nishan-Swiftui-test")
        commChannelBuilder.setMetadata(["meta": "data"])
        
        Task {
            let channel = try? await channelRepo.createChannel(with: commChannelBuilder)
            self.createdChannel = channel
            
            print("Channel Id: \(channel?.channelId)")
            print("Channel Created!")
        }
    }
    
    @MainActor
    @objc func updateChannel() {
        guard let channelId = createdChannel?.channelId, let displayName = createdChannel?.displayName else {
            print("No channel id found")
            return
        }
        
        let updateBuilder = AmityChannelUpdateBuilder(channelId: channelId)
        updateBuilder.setDisplayName("UP-\(displayName)")
        
        Task {
            let channel = try? await channelRepo.editChannel(with: updateBuilder)
            print("Channel Updated!")
        }
    }
    
    func getNewCollection() -> AmityCollection<AmityChannel> {
        let channelQuery = AmityChannelQuery()
        channelQuery.types = [AmityChannelQueryType.community]
        channelQuery.filter = .userIsMember
        
        return channelRepo.getChannels(with: channelQuery)
    }
}


